<?php

App::uses('EvCheckoutAppController', 'EvCheckout.Controller');
App::uses('OrderStatus', 'EvCheckout.Model');
/**
 * Checkout Orders Controller
 *
 * @package EvCheckout/Orders
 */
class OrdersController extends EvCheckoutAppController {

	public $components = array(
		'Transactions.Transactions', 
		// 'Transactions.SagepayDirect'
	);

	public function beforeFilter() {

		$this->Auth->allow(array(
			'basket',
			'register'
		));

		parent::beforeFilter();

	}

	protected function _adminIndexColumns() {
		return array(
			'Order.id' => array(
				'type' => 'integer',
				'null' => false,
				'default' => null,
				'length' => (int) 11,
				'key' => 'primary',
				'label' => 'ID'
			),
			'Order.created' => array(
				'type' => 'datetime',
				'null' => true,
				'default' => null,
				'length' => null,
				'label' => 'Order Created'
			),
			'User.first_name' => array(
				'type' => 'string',
				'label' => 'First Name'
			),
			'User.last_name' => array(
				'type' => 'string',
				'label' => 'Last Name'
			),
			'User.email' => array(
				'type' => 'string',
				'label' => 'Email'
			),
			'Order.status' => array(
				'type' => 'integer',
				'label' => 'Status'
			),
			'Order.total' => array(
				'type' => 'currency',
				'label' => 'Total'
			),
			'Order.modified' => array(
				'type' => 'datetime',
				'null' => true,
				'default' => null,
				'length' => null,
				'label' => 'Last Modified'
			),
		);

	}


	protected function _adminIndexActions() {

		$actions = array(
			'Edit'
		);

		return $actions;

	}

	protected function _adminIndexToolbar() {

		return array();

	}

	/**
	 * Pagination settings for admin_index
	 *
	 * Override in your own controller to customise
	 *
	 * @return array
	 */
	protected function _adminIndexPaginate() {

		$Model = $this->{$this->modelClass};

		$query = parent::_adminIndexPaginate(); 

		$query['conditions'][$Model->alias.'.order_status_id <>'] = OrderStatus::IN_BASKET;

		$query['contain'][] = 'User';

		return $query; 

	}


	public function admin_edit($id = null) {

		//No add so redirect if $id is null 
		if($id == null) {
			
			$this->Session->setFlash(array(
				'title' => 'Cannot find order',
				'description' => 'Could not find order, please search for the order in the listing below.'
			), 'flash_fail');

			$this->redirect(array('action' => 'index')); 

		}

		if (!empty($this->request->data)) {
			$user = $this->Auth->user();
			if (!empty($user['User'])) {

				$user = $user['User'];

			}

			$this->request->data['Order']['processed_by'] = $user['id'];
		}
		

		parent::admin_edit($id);

		$this->_adminPopulateLookups();

		// get and assign the previous status, this is used afterSave to check
		// whether a unit sold increment is required on a product
		if (!empty($this->request->data['Order']['previousStatus'])) {

			$previousStatus = $this->request->data['Order']['previousStatus'];

		} else {

			$previousStatus = $this->request->data['Order']['status'];

		}
		$this->set('previousStatus', $previousStatus);
		$this->view = 'admin_form';
	}


	public function admin_resendConfirmationEmail($id) {
		$this->Order->emailConfirmation($id);
		$this->Session->setFlash('Email Sent', 'flash_success');
		return $this->redirect(array('action'=>'edit', $id));
	}


	public function admin_confirmationemail($id) {
		$order = $this->Order->readForEdit($id);
		$this->view = 'confirmation';
		$this->set(array(
			'order' => $order,
			'delayed' => false,
			'telephone' => Configure::read('SiteSetting.telephone_number'),
			'sortCode' => Configure::read('SiteSetting.bank_sortcode'),
			'accountNumber' => Configure::read('SiteSetting.bank_account')
		));
		
	}


/*
 * BASKET AND CHECKOUT METHODS  
 */
	public function basket() {

		$Model = $this->{$this->modelClass};

		$this->set('cart', $Model->getCart());

	}

	public function register() {

		$Model = $this->{$this->modelClass};

		//If user is logged in skip this step and move onto the next step
		$user = $this->Auth->user();
		
		if($user) {

			return $this->redirect(array(
				'action' => 'delivery'
			));

		}

		//Else show the login and registration page. 
		//We setup the login redirect here because if the customer chooses to login that 
		// is handled by the UsersController
		$this->Session->write('Auth.redirect', 
			Router::url(array(
				'action' => 'delivery'
			))
		);

		//User has submitted registration or login form
		if(! empty($this->request->data)) {

			//Process user registration 
			$userRegistrationData = $this->request->data; 

			//Check the data validates 
			if($Model->Customer->register($userRegistrationData)) {

				$userId = $Model->Customer->id; 

				//Sign in user and send to delivery page
				$fullUser = $Model->Customer->getUserForLogin($userId);

				$this->Auth->login($fullUser);

				return $this->redirect($this->Auth->redirect());

			} else {

				$this->set('errors', $Model->Customer->validationErrors);

			}

		}
		
	}


	public function delivery() {

		$Model = $this->{$this->modelClass};

		if(! empty($this->request->data)) {

			$userId = $this->Auth->user('User.id');
			$orderId = $this->Order->getCartId();

			if($Model->DeliveryAddress->createAddress($userId, $this->request->data, $orderId)) {

				$this->redirect(array(
					'action' => 'invoice'
				));

			}

		}

	}

	public function invoice() {

		$Model = $this->{$this->modelClass};

		$cart = $Model->getCart();
		// debug($cart); die();
		$this->set('cart', $cart);

		$this->_populateLookupsInvoice();

		if(! empty($this->request->data)) {

			// debug($this->request->data); die();

			$userId = $this->Auth->user('User.id');
			$orderId = $this->Order->getCartId();

			//Split name into first and second 
			$fullName = $this->request->data['CreditCard']['name'];
			$splitName = explode(" ", $fullName);
			$this->request->data['CreditCard']['last_name'] = array_pop($splitName);

			if(count($splitName)) {

				$this->request->data['CreditCard']['first_name'] = implode(" ", $splitName);
			
			} else {
			
				$this->request->data['CreditCard']['first_name'] = '';
			
			}


			//Either mark delivery address as invoice as well or create new address
			$invoiceAddressSaved = false;

			if($this->request->data['InvoiceAddress']['match_delivery_address'] == 1) {

				$Model->setInvoiceAddress($cart['DeliveryAddress']['id']);
				$invoiceAddressSaved = true;

			} else {

				if($Model->InvoiceAddress->createAddress($userId, $this->request->data, $orderId)) {

					$invoiceAddressSaved = true;

				}

			}

			if($invoiceAddressSaved) {

				//Check that card validates 
				$this->loadModel('EvCheckout.CreditCard');
				$this->CreditCard->set($this->request->data);
				if($this->CreditCard->validates()) {
				
					$cart = $Model->getCart(); //Get cart again as the invoice address has now been set
					$result = $this->Transactions->takePayment(
						array(
							'return' => null, 
							'cancel' => null
						), 
						array(
							'model' => 'EvCheckout.Order', 
							'model_id' => $cart['Order']['id']
						), 
						$cart['Order']['total'],
						array(

						), 
						array(
							'card' => $this->request->data['CreditCard'],
							'address' => $cart['InvoiceAddress'], 
							'user' => $this->Auth->user('User')
						), 
						$cart['Order']['user_id']
						// 'sagepay_direct'
					);

				}

			}

		}

		//If validation failed we need to clear the security number
		unset($this->request->data['CreditCard']['security_number']);

	}


	protected function _populateLookupsInvoice() {

		//Available months 
		$months = $this->_getMonthsList();
		$this->set('expiryMonths', $months);
		$this->set('startMonths', $months);
		//Expiry months
		$this->set('expiryYears', $this->_getExpiryYearsList()); 
		//Start months
		$this->set('startYears', $this->_getStartYearsList());
	
	}

	protected function _getMonthsList() {

		$months = array_map(array($this, '_padArray'), range(1, 12));
		return array_combine($months, $months);
	
	}

	protected function _getExpiryYearsList() {

		$expiryYears = range(date("y"), date("y")+10);
		$expiryYearsLong = range(date("Y"), date("Y")+10);
		return array_combine($expiryYears, $expiryYearsLong);

	}

	protected function _getStartYearsList() {

		$startYears = range(date("y"), date("y")-10);
		$startYearsLong = range(date("Y"), date("Y")-10);
		return array_combine($startYears, $startYearsLong);

	}

	/**
	 * Used to pad out month and year arrays from 1 to 01
	 * @param  mixed 	$number [description]
	 * @return string 	Padded array
	 */
	protected function _padArray($number) {
		return str_pad( $number, 2, "0", STR_PAD_LEFT);
	}


}